// -*-C++-*-
// database.h
//
//  The WorldForge Project
//  Copyright (C) 2000  The WorldForge Project
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
//  For information about WorldForge and its authors, please contact
//  the Worldforge Web Site at http://www.worldforge.org.
//

#ifndef __DATABASE_H
#define __DATABASE_H

#include "graphic.h"
#include "region.h"
#include "query.h"
#include "object.h"

class CoalDatabase
{
public:
    CoalDatabase ();
    virtual ~CoalDatabase ();

    void GetMapShape (CoalShape &shape);
    void GetMapPixelSize (int &width, int &height);
    void GetMapExtents (CoalRect &rect);
    void GetMapOrigin (CoalCoord &origin);

    CoalTile* GetTileAt (const CoalCoord &coord);
    void GetTilesIn (const CoalShape &shape, CoalTileArray &tiles);
    void GetTilesIn (const CoalRect &rect, CoalTileArray &tiles);
    void QueryTiles (const CoalQuery &query, CoalTileArray &tiles);
    void GetAllTiles (CoalTileArray &tiles);

    CoalObject* GetObjectAt (const CoalCoord &coord);
    void GetObjectsIn (const CoalShape &shape, CoalObjectArray &objects);

    void FreeTile (CoalTile *tile) { if (tile) delete tile; }
    void FreeTiles (CoalTileArray &tiles);

    // Get an array of all regions, sorted by layer.
    void GetSortedRegions (CoalRegionArray &regions);

    // Graphics API
    int GetGraphicCount ();
    bool AddGraphic (CoalGraphic *graphic);
    //bool RemoveGraphic (GID gid);
    CoalGraphic* GetGraphic (unsigned int index);

    // Area Fill API
    int GetFillCount ();
    bool AddFill (CoalFill *fill);
    //bool RemoveFill (CoalFill *fill);
    CoalFill* GetFill (unsigned int index);
    CoalFill* FindFill (string &fillname);

    // Region API
    int GetRegionCount ();
    bool AddRegion (CoalRegion *region);
    //bool RemoveRegion (CoalRegion *region);
    CoalRegion* GetRegion (unsigned int index);

    // Object API
    int GetObjectCount ();
    bool AddObject (CoalObject *object);
    //bool RemoveObject (CoalObject *object);
    CoalObject* GetObject (unsigned int index);

    void SetTileSize (CoalCoordType width, CoalCoordType height)
        { m_tilewidth = width; m_tileheight = height; }
    void GetTileSize (CoalCoordType &width, CoalCoordType &height)
        { width = m_tilewidth; height = m_tileheight; }

    void SetVerbose (bool verbose) { m_verbose = verbose; }

private:
    bool m_verbose;

    // Media
    CoalGraphicArray m_graphics;
    CoalFillArray m_fills;

    // Map contents
    CoalRegionArray m_regions;  // Polygons
    CoalObjectArray m_objects;  // Sprites

    // FIXME: Belongs in an external 2D tile-rendering engine
    CoalCoordType m_tilewidth;
    CoalCoordType m_tileheight;
};

#endif  // __DATABASE_H
