// -*-C++-*-
// graphic.h
//
//  The WorldForge Project
//  Copyright (C) 2000  The WorldForge Project
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
//  For information about WorldForge and its authors, please contact
//  the Worldforge Web Site at http://www.worldforge.org.
//

// Generic container for a single graphic image.

#ifndef __GRAPHIC_H
#define __GRAPHIC_H

#include <stdio.h>  // TEMP

#include <stl.h>
#include <string>

#include "coord.h"

typedef unsigned long GID;  // Arbitrary graphics ID

class CoalColor
{
public:
    CoalColor () { red = 0; green = 0; blue = 0; }
    CoalColor (int r, int g, int b) { red = r; blue = b; green = g; }

    CoalColor& operator= (const CoalColor &copy)
        { red = copy.red; blue = copy.blue; green = copy.green; return *this; }

public:
    int red;
    int green;
    int blue;
};

// Define the various types of graphical media that a CoalGraphic can
// hold.  Can add more if necessary (mostly used by client apps).
typedef enum
{
    COAL_MEDIA_ISO_TILE = 1 << 0,
    COAL_MEDIA_MESH =     1 << 1,

    COAL_MEDIA_ALL = COAL_MEDIA_ISO_TILE | COAL_MEDIA_MESH
} CoalMediaType;

class CoalGraphic
{
public:
    CoalGraphic () { Init (); }
    CoalGraphic (string desc) { Init (); desc = desc; }
    virtual ~CoalGraphic () {}

    void Init () { id = 0; xdata = NULL; }
public:
    GID id;       // Arbitrary ID, for tracking purposes
    CoalMediaType type;  // Type of object (good for queries)
    string filename;
    string name;  // Short name of graphic object
    string desc;  // Text description (for end user)
    CoalColor bkgdcolor; // Color for bird's eye radar maps
    CoalCoord offset; // Location of focus point inside image
    void *xdata;  // Extra field for misc data (used by loaders)
};

class CoalPolyGraphic : public CoalGraphic
{
public:
    CoalPolyGraphic () { id = 0; }
    CoalPolyGraphic (string desc) { id = 0; desc = desc; }
    virtual ~CoalPolyGraphic () {}

public:
    string altfiles[8];
};

// Graphic with a coordinate
class CoalTile
{
public:
    CoalTile () {}
    CoalTile (CoalGraphic *g) { graphic = g; }
    CoalTile (CoalGraphic *g, CoalCoord &c)
        { graphic = g; coord = c; }
    virtual ~CoalTile () {}

public:
    CoalGraphic *graphic;
    CoalCoord coord;

    // Do we want an operator=() method?...
};

typedef map< GID, CoalGraphic*, less<GID> > CoalGraphicMap;
typedef CoalGraphicMap::value_type CoalGraphicMapItem;
typedef CoalGraphicMap::iterator CoalGraphicMapIter;

typedef vector<CoalGraphic*> CoalGraphicArray;
typedef vector<CoalTile*> CoalTileArray;

#endif  // __GRAPHIC_H
