// -*-C++-*-
// mimloader.h
//
//  The WorldForge Project
//  Copyright (C) 2000  The WorldForge Project
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
//  For information about WorldForge and its authors, please contact
//  the Worldforge Web Site at http://www.worldforge.org.
//

// Load MIM files and answer queries about its contents.

// The current implementation of CoalMimLoader is pretty lean and
// minimal.  (I don't even remember if it works at all yet.)  If you
// want to work on filling it out, so that COAL can parse MIM-style
// map files, drop me (John R. Sheets <dusk@ravendusk.org>) a line,
// and I'll give you a hand.  CoalBladeLoader is another fully implemented
// XML parser, using gnome-xml, which should give you a good idea of
// how to proceed.  See bladeloader.h and bladeloader.cpp.

#ifndef __MIMLOADER_H
#define __MIMLOADER_H

// gnome-xml includes
#include <tree.h>
#include <parser.h>

#include "maploader.h"

class CoalMimLoader : public CoalMapLoader
{
public:
    CoalMimLoader (CoalDatabase &db) : CoalMapLoader (db)
        { m_nextGID = 1; }  // 0 is invalid
    virtual ~CoalMimLoader () { CleanupLoader (); }

    virtual bool LoadMap (const char *mapfile);
    virtual bool LoadMap (const char *mapbuffer, int buffersize);
    virtual bool SaveMap (const char *mapfile, int save_flags = 0,
                          CoalShape *partial_map = NULL)
        { return false; }
    virtual bool CleanupLoader ();

private:
    // Convenience functions
    bool IsElement (const xmlChar *nodeName, const char *name);
    int Trim(string &str);
    string GetElementContents (const xmlNodePtr node);

    void ParseColor (const string &colortext, CoalColor &color);
    void Parse2DProperties (const string &proptext, CoalPolyGraphic *graphic);

    void ParseMIM ();
    void ParseMIMEntity (xmlNodePtr entityNode);
    void ParseMIMRegion (xmlNodePtr regionNode);
    void ParseMIMPolygon (xmlNodePtr polygonNode);
    void ParseMIMObject (xmlNodePtr objectNode);

private:
    xmlDocPtr m_xmldoc;
    CoalGraphicMap m_graphics;
    int m_nextGID;
};

#endif  // __MIMLOADER_H
