// -*-C++-*-
// query.h
//
//  The WorldForge Project
//  Copyright (C) 2000  The WorldForge Project
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
//  For information about WorldForge and its authors, please contact
//  the Worldforge Web Site at http://www.worldforge.org.
//

#ifndef __QUERY_H
#define __QUERY_H

#include <stl.h>
#include <string>

using namespace std;

class CoalRange
{
public:
    CoalRange () { SetRange (0, 1); }
    CoalRange (int start, int end) { SetRange (start, end); }
    virtual ~CoalRange () {}

    void SetStart (int start) { m_start = start; }
    void SetEnd (int end) { m_end = end; }
    void SetRange (int start, int end)
	{ SetStart (start); SetEnd (end); }
    void SetRange (const CoalRange &newrange)
	{ SetRange (newrange.GetStart(), newrange.GetEnd()); }

    CoalRange& operator= (const CoalRange &copy)
	{ SetRange (copy); return *this; }

    int GetStart () const { return m_start; }
    int GetEnd () const { return m_end; }

    bool IsInRange (int point)
	{ return (point >= m_start && point <= m_end); }

private:
    int m_start;
    int m_end;
};

class QueryParameter : public CoalRange
{
public:
    QueryParameter () { SetRange (0, 1); }
    QueryParameter (int start, int end)
        { SetRange (start, end); }
    QueryParameter (int start, int end, string ptype)
        { SetRange (start, end); SetParamType (ptype); }

    ~QueryParameter () {}

    void SetParamType (string ptype)
        { m_paramtype = ptype; }

private:
    string m_paramtype;
};

typedef void* CoalQueryDataType;

class CoalQueryData
{
public:
    CoalQueryData () {}
    virtual ~CoalQueryData () {}

    int GetCount () { return m_data.size (); }
    CoalQueryDataType GetItem (int index);
    void AppendItem (CoalQueryDataType item);
    void RemoveItem (int index, CoalQueryDataType item);

private:
    vector<CoalQueryDataType> m_data;
};

/*
 * The Query system below is a total mess.  The final version
 * will look nothing like this, I imagine.
 */
class CoalQuery
{
public:
    CoalQuery () {}
    virtual ~CoalQuery () {}

    // Convenience functions for setting up query
    void SetExtents (int start, int end)
	{ m_extents.SetRange (start, end); }
    bool QueryAll ();
    bool QueryBefore (int bound);
    bool QueryAfter (int bound);
    bool QueryBetween (CoalRange &range);
    bool QueryBetween (int bound1, int bound2);

    // Get the current state of the query
    void GetQueryParameters (QueryParameter &param);

    // Public data member, the core of the query
    CoalQueryData data;

private:
    CoalRange m_extents;  // Maximum legal range for query
    CoalRange m_query;  // FIXME: This should be a vector<>
};

#endif  // __QUERY_H
